import jinja2
import os
from base64 import b64encode
from presenters.base_presenter import BasePresenter
from taranisng.schema.parameter import Parameter, ParameterType

os.chdir("../taranis-ng-presenters/")


class TEXTPresenter(BasePresenter):
    type = "TEXT_PRESENTER"
    name = "TEXT Presenter"
    description = "Presenter for generating text documents"

    parameters = [
        Parameter(0, "TEXT_TEMPLATE_PATH", "TEXT template with its path", "Path of text template file",
                  ParameterType.STRING)
    ]

    parameters.extend(BasePresenter.parameters)

    def generate(self, presenter_input):

        try:
            head, tail = os.path.split(presenter_input.parameter_values_map['TEXT_TEMPLATE_PATH'])

            data = BasePresenter.generate_report_data_map(presenter_input)

            env = jinja2.Environment(loader=jinja2.FileSystemLoader(head))

            output_text = env.get_template(tail).render(data=data).encode()

            base64_bytes = b64encode(output_text)

            data = base64_bytes.decode('UTF-8')

            presenter_output = {
                'mime_type': 'text/plain',
                'data': data
            }

            return presenter_output
        except Exception as error:
            BasePresenter.print_exception(self, error)
